@@ -18,6 +18,7 @@
 
 #include "rrd_strtod.h"
 #include "rrd_tool.h"
+#include "fnv.h"
 
 #ifndef HAVE_G_REGEX_NEW
 #ifdef HAVE_PCRE_COMPILE
@@ -57,9 +58,7 @@ static int positive_mod(int a, int b);
 static void init_mapping(mapping_t *mapping);
 static void free_mapping(mapping_t *mapping);
 
-unsigned long FnvHash(
-    const char *str);
-void      parseGENERIC_DS(
+static void parseGENERIC_DS(
     const char *def,
     ds_def_t *ds_def);
 
@@ -238,6 +237,9 @@ done:
         free(template);
         template = NULL;
     }
+    if (opt_daemon != NULL) {
+    	free(opt_daemon);
+    }
     return rc;
 }
 
@@ -309,7 +311,7 @@ int parseDS(const char *def,
 #ifdef HAVE_G_REGEX_NEW    
     GError *gerr = NULL;
     GRegex *re = g_regex_new(DS_RE, G_REGEX_EXTENDED, 0, &gerr);
-    GMatchInfo *mi;
+    GMatchInfo *mi = NULL;
 
     if (gerr != NULL) {
         rrd_set_error("cannot compile RE: %s", gerr->message);
@@ -381,11 +383,11 @@ int parseDS(const char *def,
     case DST_DERIVE:
     case DST_DCOUNTER:
     case DST_DDERIVE:
-        strncpy(ds_def->dst, dst_tmp, DST_SIZE);
+        strncpy(ds_def->dst, dst_tmp, DST_SIZE - 1);
 	parseGENERIC_DS(dst_args, ds_def);
 	break;
     case DST_CDEF:
-	strncpy(ds_def->dst, dst_tmp, DST_SIZE);
+	strncpy(ds_def->dst, dst_tmp, DST_SIZE - 1);
         parseCDEF_DS(dst_args, ds_def, key_hash, lookup);
 	break;
     default:
@@ -444,7 +446,7 @@ int parseRRA(const char *def,
              const char **require_version) {
     char     *argvcopy;
     char     *tokptr = "";
-    unsigned short token_idx, error_flag, period = 0;
+    unsigned short token_idx, period = 0;
     int       cf_id = -1;
     int       token_min = 4;
     const char *parsetime_error = NULL;
@@ -454,14 +456,14 @@ int parseRRA(const char *def,
 
     argvcopy = strdup(def);
     char *token = strtok_r(&argvcopy[4], ":", &tokptr);
-    token_idx = error_flag = 0;
+    token_idx = 0;
             
     while (token != NULL) {
 	switch (token_idx) {
 	case 0:
 	    if (sscanf(token, CF_NAM_FMT, rra_def->cf_nam) != 1)
 		rrd_set_error("Failed to parse CF name");
-	    cf_id = cf_conv(rra_def->cf_nam);
+	    cf_id = rrd_cf_conv(rra_def->cf_nam);
 	    switch (cf_id) {
 	    case CF_MHWPREDICT:
                 if (*require_version == NULL || atoi(*require_version) < atoi(RRD_VERSION4)) {
@@ -512,7 +514,7 @@ int parseRRA(const char *def,
 	    rra_def->pdp_cnt = 1;
 	    break;
 	case 1:
-	    switch (cf_conv
+	    switch (rrd_cf_conv
 		    (rra_def->cf_nam)) {
 	    case CF_HWPREDICT:
 	    case CF_MHWPREDICT:
@@ -534,7 +536,7 @@ int parseRRA(const char *def,
 	    }
 	    break;
 	case 2:
-	    switch (cf_conv
+	    switch (rrd_cf_conv
 		    (rra_def->cf_nam)) {
 	    case CF_HWPREDICT:
 	    case CF_MHWPREDICT:
@@ -578,7 +580,7 @@ int parseRRA(const char *def,
 	    }
 	    break;
 	case 3:
-	    switch (cf_conv(rra_def->cf_nam)) {
+	    switch (rrd_cf_conv(rra_def->cf_nam)) {
 	    case CF_HWPREDICT:
 	    case CF_MHWPREDICT:
 	        if (rrd_strtodbl(token, NULL, &tmpdbl, NULL) != 2
@@ -630,7 +632,7 @@ int parseRRA(const char *def,
 	    }
 	    break;
 	case 4:
-	    switch (cf_conv(rra_def->cf_nam)) {
+	    switch (rrd_cf_conv(rra_def->cf_nam)) {
 	    case CF_FAILURES:
 		/* specifies the index (1-based) of CF_DEVSEASONAL array
 		 * associated with this CF_DEVFAILURES array. */
@@ -725,8 +727,8 @@ rra_def_t *handle_dependent_rras(rra_def_t *rra_def_array,
     rra_def_t *rra_def = rra_def_array + (*rra_cnt-1);
 
     /* should we create CF_SEASONAL, CF_DEVSEASONAL, and CF_DEVPREDICT? */
-    if ((cf_conv(rra_def->cf_nam) == CF_HWPREDICT
-	 || cf_conv(rra_def->cf_nam) == CF_MHWPREDICT)
+    if ((rrd_cf_conv(rra_def->cf_nam) == CF_HWPREDICT
+	 || rrd_cf_conv(rra_def->cf_nam) == CF_MHWPREDICT)
 	&& rra_def->par[RRA_dependent_rra_idx].u_cnt == INT_MAX) {
 	rra_def->par[RRA_dependent_rra_idx].u_cnt = *rra_cnt-1;
 #ifdef DEBUG
@@ -858,6 +860,7 @@ int rrd_create_r2(
         
         if (rrd.ds_def == NULL || rrd.rra_def == NULL) {
             rrd_set_error("cannot allocate memory");
+	    rrd_close(tf);
             goto done;
         }
 
@@ -948,7 +951,7 @@ int rrd_create_r2(
     // parsing went well. ONLY THEN are we allowed to produce
     // additional side effects.
     if (require_version != NULL) {
-        strcpy(rrd.stat_head->version, require_version);
+        strncpy(rrd.stat_head->version, require_version, 5);
     }
 
     if (rrd.stat_head->rra_cnt < 1) {
@@ -1023,7 +1026,7 @@ done:
     return rc;
 }
 
-void parseGENERIC_DS(
+static void parseGENERIC_DS(
     const char *def,
     ds_def_t *ds_def)
 {
@@ -1169,7 +1172,7 @@ rra_def_t * create_hw_contingent_rras(rra_def_t *rra_defs,
 void init_cdp(const rrd_t *rrd, const rra_def_t *rra_def, const pdp_prep_t *pdp_prep, cdp_prep_t *cdp_prep)
 {
 
-    switch (cf_conv(rra_def->cf_nam)) {
+    switch (rrd_cf_conv(rra_def->cf_nam)) {
         case CF_HWPREDICT:
         case CF_MHWPREDICT:
             init_hwpredict_cdp(cdp_prep);
@@ -1313,6 +1316,7 @@ done:
 int write_rrd(const char *outfilename, rrd_t *out) {
     int rc = -1;
     char *tmpfilename = NULL;
+    mode_t saved_umask;
 
     /* write out the new file */
 #ifdef HAVE_LIBRADOS
@@ -1337,7 +1341,10 @@ int write_rrd(const char *outfilename, rrd_t *out) {
 	strcpy(tmpfilename, outfilename);
 	strcat(tmpfilename, "XXXXXX");
 	
+	/* fix CWE-377 */
+	saved_umask = umask(S_IRUSR|S_IWUSR);
 	int tmpfd = mkstemp(tmpfilename);
+	umask(saved_umask);
 	if (tmpfd < 0) {
 	    rrd_set_error("Cannot create temporary file");
 	    goto done;
@@ -1764,10 +1771,10 @@ static rrd_value_t prefill_finish(rra_def_t UNUSED(*rra_def), enum cf_en current
 }
 
 static int order_candidates(candidate_t *a, candidate_t *b, const candidate_t UNUSED(*target)) {
-    enum cf_en acf = cf_conv(a->rra->cf_nam);
-    enum cf_en bcf = cf_conv(b->rra->cf_nam);
+    enum cf_en acf = rrd_cf_conv(a->rra->cf_nam);
+    enum cf_en bcf = rrd_cf_conv(b->rra->cf_nam);
 
-    enum cf_en tcf = cf_conv(target->rra->cf_nam);
+    enum cf_en tcf = rrd_cf_conv(target->rra->cf_nam);
     
     int astep = a->rrd->stat_head->pdp_step;
     int bstep = b->rrd->stat_head->pdp_step;
@@ -1802,9 +1809,9 @@ static int order_candidates(candidate_t *a, candidate_t *b, const candidate_t UN
 
 /* select AVERAGE and same CF RRAs. */
 static int select_create_candidates(const rra_def_t *tofill, const rra_def_t *maybe) {
-    enum cf_en cf = cf_conv(maybe->cf_nam);
+    enum cf_en cf = rrd_cf_conv(maybe->cf_nam);
     if (cf == CF_AVERAGE) return 1;
-    if (cf == cf_conv(tofill->cf_nam)) return 1;
+    if (cf == rrd_cf_conv(tofill->cf_nam)) return 1;
     return 0;
 }
 
@@ -2027,7 +2034,7 @@ static void prefill_cdp_prep(candidate_t *target,
     
     (target->cdp + target->extra.l)->scratch[CDP_unkn_pdp_cnt].u_cnt = 0;
 
-    enum cf_en tcf = cf_conv(target->rra->cf_nam);
+    enum cf_en tcf = rrd_cf_conv(target->rra->cf_nam);
     time_t t;
 
     rra_def_t *cdp_rra = rrd->rra_def + cdp_rra_index;
@@ -2204,7 +2211,7 @@ static long rra_for_cdp_prefilling(rrd_t *rrd, int *added)
         original_total_rows += rra->row_cnt;
         
         largest_pdp = max(largest_pdp, rra->pdp_cnt);
-        if (rra->pdp_cnt == 1 && cf_conv(rra->cf_nam) == CF_AVERAGE) {
+        if (rra->pdp_cnt == 1 && rrd_cf_conv(rra->cf_nam) == CF_AVERAGE) {
             if (average_1_pdp_rra == NULL) {
                 average_1_pdp_rra = rra;
                 found_rra_index = rra_index;
@@ -2285,8 +2292,8 @@ static void remove_temporary_rra_for_cdp_prefilling(rrd_t *rrd, long added_index
 }
 
 static int cdp_match(const rra_def_t *tofill, const rra_def_t *maybe) {
-    enum cf_en mcf = cf_conv(maybe->cf_nam);
-    if (cf_conv(tofill->cf_nam) == mcf &&
+    enum cf_en mcf = rrd_cf_conv(maybe->cf_nam);
+    if (rrd_cf_conv(tofill->cf_nam) == mcf &&
         tofill->pdp_cnt == maybe->pdp_cnt) {
         return 1;
     }
@@ -2324,7 +2331,7 @@ static int rrd_prefill_data(rrd_t *rrd, const GList *sources, mapping_t *mapping
         candidate_t target = {
             .rrd = rrd,
             .rra = rra_def,
-            .rra_cf = cf_conv(rra_def->cf_nam),
+            .rra_cf = rrd_cf_conv(rra_def->cf_nam),
             .rra_index = rra_index,
             .values = rrd->rrd_value + rrd->stat_head->ds_cnt * total_rows,
             .cdp = rrd->cdp_prep + rrd->stat_head->ds_cnt * rra_index,
@@ -2388,7 +2395,7 @@ static int rrd_prefill_data(rrd_t *rrd, const GList *sources, mapping_t *mapping
         candidate_t target = {
             .rrd = rrd,
             .rra = rrd->rra_def + rra_index,
-            .rra_cf = cf_conv(rrd->rra_def[rra_index].cf_nam),
+            .rra_cf = rrd_cf_conv(rrd->rra_def[rra_index].cf_nam),
             .rra_index = rra_index,
             .values = rrd->rrd_value + rrd->stat_head->ds_cnt * total_rows,
             .cdp = rrd->cdp_prep + rrd->stat_head->ds_cnt * rra_index,
