@@ -51,17 +51,6 @@
 extern char *tzname[2];
 #endif
 
-//Local prototypes
-size_t rrd_dump_opt_cb_fileout(
-    const void *data,
-    size_t len,
-    void *user);
-
-int rrd_dump_opt_r(
-    const char *filename,
-    char *outname,
-    int opt_noheader);
-
 int rrd_dump_cb_r(
     const char *filename,
     int opt_header,
@@ -221,7 +210,7 @@ int rrd_dump_cb_r(
         /* support for RRA parameters */
         CB_PUTS("\t\t<params>\n");
 
-        switch (cf_conv(rrd.rra_def[i].cf_nam)) {
+        switch (rrd_cf_conv(rrd.rra_def[i].cf_nam)) {
         case CF_HWPREDICT:
         case CF_MHWPREDICT:
             CB_FMTS("\t\t<hw_alpha>%0.10e</hw_alpha>\n",
@@ -305,7 +294,7 @@ int rrd_dump_cb_r(
                 CB_FMTS("\t\t\t<secondary_value>%0.10e</secondary_value>\n", value);
             }
 
-            switch (cf_conv(rrd.rra_def[i].cf_nam)) {
+            switch (rrd_cf_conv(rrd.rra_def[i].cf_nam)) {
             case CF_HWPREDICT:
             case CF_MHWPREDICT:
                 value = rrd.cdp_prep[i * rrd.stat_head->ds_cnt + ii].
@@ -422,7 +411,7 @@ int rrd_dump_cb_r(
             now = (rrd.live_head->last_up
                    - rrd.live_head->last_up
                    % (rrd.rra_def[i].pdp_cnt * rrd.stat_head->pdp_step))
-                + (timer * rrd.rra_def[i].pdp_cnt * rrd.stat_head->pdp_step);
+                + (timer * (long)rrd.rra_def[i].pdp_cnt * (long)rrd.stat_head->pdp_step);
 
             timer++;
 #if HAVE_STRFTIME
@@ -465,7 +454,7 @@ err_out:
 
 }
 
-size_t rrd_dump_opt_cb_fileout(
+static size_t rrd_dump_opt_cb_fileout(
     const void *data,
     size_t len,
     void *user)
@@ -540,8 +529,9 @@ int rrd_dump(
     while ((opt = optparse_long(&options, longopts, NULL)) != -1) {
         switch (opt) {
         case 'd':
-            if (opt_daemon != NULL)
+            if (opt_daemon != NULL) {
                     free (opt_daemon);
+            }
             opt_daemon = strdup(options.optarg);
             if (opt_daemon == NULL)
             {
@@ -569,6 +559,9 @@ int rrd_dump(
                           "[--no-header|-n]\n"
                           "[--daemon|-d address]\n"
                           "file.rrd [file.xml]", options.argv[0]);
+            if (opt_daemon != NULL) {
+            	free(opt_daemon);
+            }
             return (-1);
             break;
         }
@@ -579,11 +572,16 @@ int rrd_dump(
                       "[--no-header|-n]\n"
                       "[--daemon|-d address]\n"
                        "file.rrd [file.xml]", options.argv[0]);
+        if (opt_daemon != NULL) {
+            free(opt_daemon);
+        }
         return (-1);
     }
 
     rc = rrdc_flush_if_daemon(opt_daemon, options.argv[options.optind]);
-    if (opt_daemon) free(opt_daemon);
+    if (opt_daemon != NULL) {
+    	free(opt_daemon);
+    }
     if (rc) return (rc);
 
     if ((options.argc - options.optind) == 2) {
