@@ -250,7 +250,10 @@ static int _sql_fetchrow(struct sql_table_helper* th,time_t *timestamp, rrd_valu
     /* calculate the table to use next */
     th->table_start=th->table_next;
     th->table_next=_find_next_separator(th->table_start,'+');
-    _inline_unescape(th->table_start);
+    if (_inline_unescape(th->table_start)) {
+      _sql_close(th);
+      return -1;
+    }
     /* and prepare FULL SQL Statement */
     if (ordered) {
       snprintf(sql,sizeof(sql)-1,"SELECT %s as rrd_time, %s as rrd_value FROM %s WHERE %s ORDER BY %s",
@@ -310,7 +313,7 @@ static char* _find_next_separator_twice(char*start,char separator) {
       /* and return the pointer past the current one*/
       return (found+2);
     }
-    /* find next occurance */
+    /* find next occurence */
     found=strchr(found+1,separator);
   }
   /* not found, so return NULL */
@@ -353,7 +356,7 @@ static int _inline_unescape (char* string) {
     src++;
     if (c == '%') {
       if (*src == '%') { 
-	/* increase src pointer by 1 skiping second % */
+	/* increase src pointer by 1 skipping second % */
 	src+=1;
       } else {
 	/* try to calculate hex value from the next 2 values*/
@@ -363,12 +366,12 @@ static int _inline_unescape (char* string) {
 	  return(1);
 	}
 	h2=_hexcharhelper(*(src+1));
-	if (h1 == (char)-1) {
+	if (h2 == (char)-1) {
 	  rrd_set_error("string escape error at: %s\n",string);
 	  return(1);
 	}
 	c=h2+(h1<<4);
-	/* increase src pointer by 2 skiping 2 chars */
+	/* increase src pointer by 2 skipping 2 chars */
 	src+=2;
       } 
     }
@@ -435,7 +438,8 @@ rrd_fetch_fn_libdbi(
   separator=filename[3];
 
   /* copy filename for local modifications during parsing */
-  strncpy(filenameworkcopy,filename+5,sizeof(filenameworkcopy));
+  strncpy(filenameworkcopy,filename+5,sizeof(filenameworkcopy) - 1);
+  filenameworkcopy[sizeof(filenameworkcopy) - 1] = '\0';
 
   /* get the driver */
   table_help.dbdriver=tmpptr;
@@ -526,7 +530,11 @@ rrd_fetch_fn_libdbi(
     } else if (*sqlargs==0) { /* ignore empty */
     } else { /* else add to where string */
       if (where[0]) {strcat(where," AND ");}
-      strcat(where,sqlargs);
+      if (strlen(where) + strlen(sqlargs) >= sizeof(where)) {
+        rrd_set_error("argument too long (exceeded %d characters)", sizeof(where) - 1);
+        return -1;
+      }
+      strncat(where,sqlargs, sizeof(where) - strlen(sqlargs) - 1);
     }
     /* and continue loop with next pointer */
     sqlargs=nextptr;
@@ -663,7 +671,7 @@ rrd_fetch_fn_libdbi(
       }
     }
   }
-  /* and check for negativ status, pass back immediately */
+  /* and check for negative status, pass back immediately */
   if (r_status==-1) { return -1; }
 
   /* post processing */
