@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.6.0, Copyright 1997-2016 by Tobi Oetiker
+ * RRDtool 1.7.0, Copyright 1997-2017 by Tobi Oetiker
  *                Copyright by Florian Forster, 2008
  *****************************************************************************
  * rrd_lastupdate  Get the last datum entered for each DS
@@ -29,8 +29,9 @@ int rrd_lastupdate (int argc, char **argv)
     while ((opt = optparse_long(&options, longopts, NULL)) != -1) {
         switch (opt) {
         case 'd':
-            if (opt_daemon != NULL)
+            if (opt_daemon != NULL) {
                     free (opt_daemon);
+            }
             opt_daemon = strdup(options.optarg);
             if (opt_daemon == NULL)
             {
@@ -41,6 +42,9 @@ int rrd_lastupdate (int argc, char **argv)
 
         case '?':
             rrd_set_error("%s", options.errmsg);
+            if (opt_daemon != NULL) {
+            	free(opt_daemon);
+            }
             return -1;
         }
     }                   /* while (opt!=-1) */
@@ -48,11 +52,16 @@ int rrd_lastupdate (int argc, char **argv)
     if ((options.argc - options.optind) != 1) {
         rrd_set_error ("Usage: rrdtool %s [--daemon|-d <addr>] <file>",
                 options.argv[0]);
+        if (opt_daemon != NULL) {
+            free(opt_daemon);
+        }
         return (-1);
     }
 
     status = rrdc_flush_if_daemon(opt_daemon, options.argv[options.optind]);
-    if (opt_daemon) free (opt_daemon);
+    if (opt_daemon != NULL) {
+    	free (opt_daemon);
+    }
     if (status) return (-1);
 
     status = rrd_lastupdate_r(options.argv[options.optind],
