@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.6.0, Copyright 1997-2016 by Tobi Oetiker
+ * RRDtool 1.7.0, Copyright 1997-2017 by Tobi Oetiker
  *****************************************************************************
  * rrd_open.c  Open an RRD File
  *****************************************************************************
@@ -43,7 +43,7 @@
 #endif
 
 /* DEBUG 2 prints information obtained via mincore(2) */
-#define DEBUG 1
+// #define DEBUG 1
 /* do not calculate exact madvise hints but assume 1 page for headers and
  * set DONTNEED for the rest, which is assumed to be data */
 /* Avoid calling madvise on areas that were already hinted. May be beneficial if
@@ -178,6 +178,7 @@ rrd_file_t *rrd_open(
     rrd_file->pvt = malloc(sizeof(rrd_simple_file_t));
     if(rrd_file->pvt == NULL) {
         rrd_set_error("allocating rrd_simple_file for '%s'", file_name);
+        free(rrd_file);
         return NULL;
     }
     memset(rrd_file->pvt, 0, sizeof(rrd_simple_file_t));
@@ -188,7 +189,8 @@ rrd_file_t *rrd_open(
         (RRD_READONLY | RRD_READWRITE)) {
         /* Both READONLY and READWRITE were given, which is invalid.  */
         rrd_set_error("in read/write request mask");
-        exit(-1);
+        free(rrd_file);
+        return NULL;
     }
 #endif
 
@@ -278,17 +280,30 @@ rrd_file_t *rrd_open(
     } else {
         rrd_file->file_len = newfile_size;
 #ifdef HAVE_POSIX_FALLOCATE
-        if (posix_fallocate(rrd_simple_file->fd, 0, newfile_size) == 0){
-            /* if all  is well we skip the seeking below */            
+	/* man: posix_fallocate() returns zero on success,
+	 * or an error number on failure.  Note that errno is not set.
+	 */
+	int fret = posix_fallocate(rrd_simple_file->fd, 0, newfile_size);
+        if (fret) {
+            rrd_set_error("posix_fallocate '%s': %s", file_name,
+                          rrd_strerror(fret));
+            goto out_close;
+        } else {
             goto no_lseek_necessary;        
         }
 #endif
-        lseek(rrd_simple_file->fd, newfile_size - 1, SEEK_SET);
+        if (lseek(rrd_simple_file->fd, newfile_size - 1, SEEK_SET) == -1) {
+            rrd_set_error("lseek '%s': %s", file_name, rrd_strerror(errno));
+            goto out_close;
+        }
         if ( write(rrd_simple_file->fd, "\0", 1) == -1){    /* poke */
             rrd_set_error("write '%s': %s", file_name, rrd_strerror(errno));
             goto out_close;
         }
-        lseek(rrd_simple_file->fd, 0, SEEK_SET);
+        if (lseek(rrd_simple_file->fd, 0, SEEK_SET) == -1){
+            rrd_set_error("lseek '%s': %s", file_name, rrd_strerror(errno));
+            goto out_close;
+        }
     }
     no_lseek_necessary:
 #if !defined(HAVE_MMAP) && defined(HAVE_POSIX_FADVISE)
@@ -580,7 +595,7 @@ void rrd_dontneed(
     rrd_file_t *rrd_file,
     rrd_t *rrd)
 {
-    rrd_simple_file_t *rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
+    rrd_simple_file_t *rrd_simple_file;
 #if defined USE_MADVISE || defined HAVE_POSIX_FADVISE
     size_t dontneed_start;
     size_t rra_start;
@@ -594,6 +609,7 @@ void rrd_dontneed(
 #endif
 	    return;
     }
+    rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
 
 #if defined DEBUG && DEBUG > 1
     mincore_print(rrd_file, "before");
