@@ -2,7 +2,7 @@
 #define RRD_TOOL_H_3853987DDF7E4709A5B5849E5A6204F4
 
 /*****************************************************************************
- * RRDtool 1.6.0, Copyright 1997-2016 by Tobi Oetiker
+ * RRDtool 1.7.0, Copyright 1997-2017 by Tobi Oetiker
  *****************************************************************************
  * rrd_tool.h   Common Header File
  *****************************************************************************/
@@ -134,7 +134,7 @@ typedef int (*rrd_fetch_cb_t)(
 #define RRD_EXCL        (1<<5)
 #define RRD_READVALUES  (1<<6)
 
-    enum cf_en cf_conv(
+    enum cf_en rrd_cf_conv(
     const char *string);
     enum dst_en dst_conv(
     const char *string);
