@@ -49,21 +49,21 @@
 #include "rrd_modify.h"
 #include "rrd_client.h"
 
-int       set_hwarg(
+static int set_hwarg(
     rrd_t *rrd,
     enum cf_en cf,
     enum rra_par_en rra_par,
     const char *arg);
-int       set_deltaarg(
+static int set_deltaarg(
     rrd_t *rrd,
     enum rra_par_en rra_par,
     const char *arg);
-int       set_windowarg(
+static int set_windowarg(
     rrd_t *rrd,
     enum rra_par_en,
     const char *arg);
 
-int set_hwsmootharg(
+static int set_hwsmootharg(
     rrd_t *rrd,
     enum cf_en cf,
     enum rra_par_en rra_par,
@@ -116,6 +116,9 @@ int rrd_tune(
     struct optparse options;
     int opt;
 
+    /* Fix CWE-457 */
+    memset(&rrd, 0, sizeof(rrd_t));
+
     /* before we open the input RRD, we should flush it from any caching
     daemon, because we might totally rewrite it later on */
 
@@ -142,6 +145,7 @@ int rrd_tune(
     // connect to daemon (will take care of environment variable automatically)
     if (rrdc_connect(opt_daemon) != 0) {
     	rrd_set_error("Cannot connect to daemon");
+    	free(opt_daemon);
 	return 1;
     }
 
@@ -150,7 +154,7 @@ int rrd_tune(
 	opt_daemon = NULL;
     }
 
-    if (options.optind < 0 || options.optind >= options.argc) {
+    if (!options.optind || options.optind >= options.argc) {
 	// missing file name...
 	rrd_set_error("missing file name");
 	goto done;
@@ -402,10 +406,6 @@ int rrd_tune(
     }
 
     optind = handle_modify(&rrd, in_filename, options.argc, options.argv, options.optind + 1, opt_newstep);
-    if (options.optind < 0) {
-	goto done;
-    }
-    
     rc = 0;
 done:
     if (in_filename && rrdc_is_any_connected()) {
@@ -428,7 +428,7 @@ done:
     return rc;
 }
 
-int set_hwarg(
+static int set_hwarg(
     rrd_t *rrd,
     enum cf_en cf,
     enum rra_par_en rra_par,
@@ -451,7 +451,7 @@ int set_hwarg(
     }
     /* does the appropriate RRA exist?  */
     for (i = 0; i < rrd->stat_head->rra_cnt; ++i) {
-        if (cf_conv(rrd->rra_def[i].cf_nam) == cf) {
+        if (rrd_cf_conv(rrd->rra_def[i].cf_nam) == cf) {
             rra_idx = i;
             break;
         }
@@ -466,7 +466,7 @@ int set_hwarg(
     return 0;
 }
 
-int set_hwsmootharg(
+static int set_hwsmootharg(
     rrd_t *rrd,
     enum cf_en cf,
     enum rra_par_en rra_par,
@@ -491,7 +491,7 @@ int set_hwsmootharg(
     }
     /* does the appropriate RRA exist?  */
     for (i = 0; i < rrd->stat_head->rra_cnt; ++i) {
-        if (cf_conv(rrd->rra_def[i].cf_nam) == cf) {
+        if (rrd_cf_conv(rrd->rra_def[i].cf_nam) == cf) {
             rra_idx = i;
             break;
         }
@@ -506,7 +506,7 @@ int set_hwsmootharg(
     return 0;
 }
 
-int set_deltaarg(
+static int set_deltaarg(
     rrd_t *rrd,
     enum rra_par_en rra_par,
     const char *arg)
@@ -528,7 +528,7 @@ int set_deltaarg(
 
     /* does the appropriate RRA exist?  */
     for (i = 0; i < rrd->stat_head->rra_cnt; ++i) {
-        if (cf_conv(rrd->rra_def[i].cf_nam) == CF_FAILURES) {
+        if (rrd_cf_conv(rrd->rra_def[i].cf_nam) == CF_FAILURES) {
             rra_idx = i;
             break;
         }
@@ -543,7 +543,7 @@ int set_deltaarg(
     return 0;
 }
 
-int set_windowarg(
+static int set_windowarg(
     rrd_t *rrd,
     enum rra_par_en rra_par,
     const char *arg)
@@ -561,7 +561,7 @@ int set_windowarg(
     }
     /* does the appropriate RRA exist?  */
     for (i = 0; i < rrd->stat_head->rra_cnt; ++i) {
-        if (cf_conv(rrd->rra_def[i].cf_nam) == CF_FAILURES) {
+        if (rrd_cf_conv(rrd->rra_def[i].cf_nam) == CF_FAILURES) {
             rra_idx = i;
             break;
         }
