@@ -61,13 +61,7 @@ static int gettimeofday(
 #endif
 
 /* FUNCTION PROTOTYPES */
-
-int       rrd_update_r(
-    const char *filename,
-    const char *tmplt,
-    int argc,
-    const char **argv);
-int       _rrd_updatex(
+static int _rrd_updatex(
     const char *filename,
     const char *tmplt,
     int extra_flags,
@@ -692,6 +686,9 @@ int rrd_update(
     while ((opt = optparse_long(&options,longopts,NULL)) != -1) {
         switch (opt) {
         case 't':
+            if (tmplt != NULL) {
+            	free(tmplt);
+            }
             tmplt = strdup(options.optarg);
             break;
 
@@ -700,8 +697,9 @@ int rrd_update(
             break;
 
         case 'd':
-            if (opt_daemon != NULL)
+            if (opt_daemon != NULL) {
                 free (opt_daemon);
+            }
             opt_daemon = strdup (options.optarg);
             if (opt_daemon == NULL)
             {
@@ -815,7 +813,7 @@ int rrd_updatex_v_r(
     return _rrd_updatex(filename, tmplt, extra_flags, argc, argv, pcdp_summary);
 }
 
-int _rrd_updatex(
+static int _rrd_updatex(
     const char *filename,
     const char *tmplt,
     int extra_flags,
@@ -1774,7 +1772,7 @@ static int update_all_cdp_prep(
 
     rra_start = rra_begin;
     for (rra_idx = 0; rra_idx < rrd->stat_head->rra_cnt; rra_idx++) {
-        current_cf = cf_conv(rrd->rra_def[rra_idx].cf_nam);
+        current_cf = rrd_cf_conv(rrd->rra_def[rra_idx].cf_nam);
         start_pdp_offset =
             rrd->rra_def[rra_idx].pdp_cnt -
             proc_pdp_cnt % rrd->rra_def[rra_idx].pdp_cnt;
@@ -2019,7 +2017,7 @@ static void initialize_cdp_val(
             fprintf(stderr,
                     "RRA %lu, DS %lu, both CDP_val and pdp_temp are DNAN!",
                     i, ii);
-            exit(-1);
+            break;
         }
 #endif
 #endif
@@ -2037,7 +2035,7 @@ static void initialize_cdp_val(
             fprintf(stderr,
                     "RRA %lu, DS %lu, both CDP_val and pdp_temp are DNAN!", i,
                     ii);
-            exit(-1);
+            break;
         }
 #endif
 #endif
@@ -2209,7 +2207,7 @@ static int update_aberrant_cdps(
         rra_start = rra_begin;
         for (rra_idx = 0; rra_idx < rrd->stat_head->rra_cnt; rra_idx++) {
             if (rrd->rra_def[rra_idx].pdp_cnt == 1) {
-                current_cf = cf_conv(rrd->rra_def[rra_idx].cf_nam);
+                current_cf = rrd_cf_conv(rrd->rra_def[rra_idx].cf_nam);
                 if (current_cf == CF_SEASONAL || current_cf == CF_DEVSEASONAL) {
                     if (scratch_idx == CDP_primary_val) {
                         lookup_seasonal(rrd, rra_idx, rra_start, rrd_file,
@@ -2384,8 +2382,8 @@ static int smooth_all_rras(
     unsigned long rra_idx;
 
     for (rra_idx = 0; rra_idx < rrd->stat_head->rra_cnt; ++rra_idx) {
-        if (cf_conv(rrd->rra_def[rra_idx].cf_nam) == CF_DEVSEASONAL ||
-            cf_conv(rrd->rra_def[rra_idx].cf_nam) == CF_SEASONAL) {
+        if (rrd_cf_conv(rrd->rra_def[rra_idx].cf_nam) == CF_DEVSEASONAL ||
+            rrd_cf_conv(rrd->rra_def[rra_idx].cf_nam) == CF_SEASONAL) {
 #ifdef DEBUG
             fprintf(stderr, "Running smoother for rra %lu\n", rra_idx);
 #endif
